@@ -60,8 +60,10 @@ static GMainLoop* loop = NULL;
 /* The URL of the crash database. */
 static char* crash_db_url = NULL;
 
-/*  The system UUID, taken from the DMI tables and SHA-512 hashed */
-static char sha512_system_uuid[HASHLEN + 1] = {0};
+/*  The database identifier. Either:
+ *  - The system UUID, taken from the DMI tables and SHA-512 hashed
+ *  - The MAC address of the first non-loopback device, SHA-512 hashed */
+static char* whoopsie_identifier = NULL;
 
 /* The URL for sending the initial crash report */
 static char* crash_db_submit_url = NULL;
@@ -101,6 +103,7 @@ static const char* acceptable_fields[] = {
     "Uname",
     "Architecture",
     "NonfreeKernelModules",
+    "LiveMediaBuild",
     /* add_user_info */
     "UserGroups",
     /* add_proc_info */
@@ -124,6 +127,12 @@ static const char* acceptable_fields[] = {
     "CoreDump",
     /* add_kernel_crash_info */
     "VmCore",
+    /* We use package-from-proposed tag to determine if a problem is occuring
+     * in the release-proposed pocket. */
+    "Tags",
+    /* We need the OopsText field to be able to generate a crash signature from
+     * KernelOops problems */
+    "OopsText",
 
     /* Fields we do not care about: */
 
@@ -141,7 +150,11 @@ static const char* acceptable_fields[] = {
 static gboolean
 is_acceptable_field (const char* field)
 {
-    const char** p = acceptable_fields;
+    const char** p;
+
+    g_return_val_if_fail (field, FALSE);
+
+    p = acceptable_fields;
     while (*p) {
         if (strcmp (*p, field) == 0)
             return TRUE;
@@ -180,6 +193,8 @@ append_key_value (gpointer key, gpointer value, gpointer bson_string)
      * need it */
     if (!strcmp ("CoreDump", k))
         return TRUE;
+    if (!strcmp ("VmCore", k))
+        return TRUE;
 
     return bson_append_string (str, k, v) != BSON_ERROR;
 }
@@ -195,6 +210,9 @@ server_response (char* ptr, size_t size, size_t nmemb, void* s)
 void
 split_string (char* head, char** tail)
 {
+    g_return_if_fail (head);
+    g_return_if_fail (tail);
+
     *tail = strchr (head, ' ');
     if (*tail) {
         **tail = '\0';
@@ -216,7 +234,7 @@ bsonify (GHashTable* report, bson* b, const char** bson_message,
     *bson_message = NULL;
     *bson_message_len = 0;
 
-    assert (report != NULL);
+    g_return_val_if_fail (report, FALSE);
 
     bson_init (b);
     if (!bson_data (b))
@@ -246,6 +264,8 @@ upload_report (const char* message_data, int message_len, struct response_string
     long response_code = 0;
     struct curl_slist* list = NULL;
 
+    g_return_val_if_fail (message_data, -1);
+
     if ((curl = curl_easy_init ()) == NULL) {
         printf ("Couldn't init curl.\n");
         return FALSE;
@@ -312,6 +332,8 @@ parse_report (const char* report_path, gboolean full_report, GError** error)
     int value_pos;
     gboolean ignore_field = FALSE;
 
+    g_return_val_if_fail (report_path, NULL);
+
     if (g_file_test (report_path, G_FILE_TEST_IS_SYMLINK) ||
         !g_file_test (report_path, G_FILE_TEST_IS_REGULAR)) {
         g_set_error (error, g_quark_from_static_string ("whoopsie-quark"), 0,
@@ -479,8 +501,13 @@ upload_core (const char* uuid, const char* arch, const char* core_data) {
     char* crash_db_core_url = NULL;
     struct response_string s;
 
+
+    g_return_val_if_fail (uuid, FALSE);
+    g_return_val_if_fail (arch, FALSE);
+    g_return_val_if_fail (core_data, FALSE);
+
     if (asprintf (&crash_db_core_url, "%s/%s/submit-core/%s/%s",
-        crash_db_url, uuid, arch, sha512_system_uuid) < 0)
+        crash_db_url, uuid, arch, whoopsie_identifier) < 0)
         g_error ("Unable to allocate memory.");
 
     /* TODO use CURLOPT_READFUNCTION to transparently compress data with
@@ -523,6 +550,8 @@ handle_response (GHashTable* report, char* response_data)
     char* core = NULL;
     char* arch = NULL;
 
+    g_return_if_fail (report);
+
     /* Command could be CORE, which requests the core dump, BUG ######, if in a
      * development release, which points to the bug report, or UPDATE, if this
      * is fixed in an update. */
@@ -556,7 +585,7 @@ parse_and_upload_report (const char* crash_file)
     bson b[1];
     int response = 0;
 
-    g_print ("Uploading %s.\n", crash_file);
+    g_print ("Parsing %s.\n", crash_file);
     report = parse_report (crash_file, FALSE, &error);
     if (!report) {
         if (error) {
@@ -577,6 +606,7 @@ parse_and_upload_report (const char* crash_file)
         /* Do not keep trying to parse and upload this */
         success = TRUE;
     } else {
+        g_print ("Uploading %s.\n", crash_file);
         init_response_string (&s);
         response = upload_report (message_data, message_len, &s);
         if (bson_data (b))
@@ -618,6 +648,9 @@ process_existing_files (void)
     while ((file = g_dir_read_name (dir)) != NULL) {
 
         upload_file = g_build_filename ("/var/crash", file, NULL);
+        if (!upload_file)
+            continue;
+
         ext = strrchr (upload_file, '.');
         if (ext && strcmp(++ext, "upload") != 0) {
             free (upload_file);
@@ -626,10 +659,11 @@ process_existing_files (void)
 
         crash_file = change_file_extension (upload_file, ".crash");
         if (!crash_file) {
-            /* FIXME this would be bad - we'd keep uploading it */
-            if (!mark_handled (crash_file))
-                g_print ("Unable to mark report as seen (%s)\n", crash_file);
-        } else if (already_handled_report (crash_file)) {
+            free (upload_file);
+            continue;
+        }
+
+        if (already_handled_report (crash_file)) {
             if (!mark_handled (crash_file))
                 g_print ("Unable to mark report as seen (%s)\n", crash_file);
         } else if (online_state && parse_and_upload_report (crash_file)) {
@@ -813,8 +847,6 @@ int
 main (int argc, char** argv)
 {
     GError* err = NULL;
-    char* system_uuid = NULL;
-    char* mac_address = NULL;
 
     setup_signals ();
     parse_arguments (&argc, &argv);
@@ -824,34 +856,16 @@ main (int argc, char** argv)
         exit (EXIT_FAILURE);
     }
 
-    identifier_get_system_uuid (&system_uuid, &err);
+    whoopsie_identifier_generate (&whoopsie_identifier, &err);
     if (err) {
         g_print ("%s\n", err->message);
         g_error_free (err);
         err = NULL;
-    }
-    if (system_uuid) {
-        identifier_sha512 (system_uuid, sha512_system_uuid, NULL);
-        free (system_uuid);
-    } else {
-        identifier_get_mac_address (&mac_address, &err);
-        if (err) {
-            g_print ("%s\n", err->message);
-            g_error_free (err);
-            err = NULL;
-        }
-        if (mac_address) {
-            identifier_sha512 (mac_address, sha512_system_uuid, NULL);
-            free (mac_address);
-        }
-    }
-
-    if (*sha512_system_uuid != '\0') {
-        if (asprintf (&crash_db_submit_url, "%s/%s",
-            crash_db_url, sha512_system_uuid) < 0)
-            g_error ("Unable to allocate memory.");
-    } else {
         crash_db_submit_url = strdup (crash_db_url);
+
+    } else if (asprintf (&crash_db_submit_url, "%s/%s", crash_db_url,
+               whoopsie_identifier) < 0) {
+        g_error ("Unable to allocate memory.");
     }
 
     drop_privileges (&err);
